package musei;

import java.sql.*;
import java.util.*;

/**
 * Questa classe mette a disposizione i metodi per effettuare interrogazioni
 * sulla base di dati.
 *
 * @author Barbara Oliboni, Gabriele Pozzani
 * @version 1.0
 */
public class DBMS {
    /** Utente per la connessione alla base di dati. */
    private String user = "userlab200";
    /** Password associata all'utente della base di dati. */
    private String passwd = "";
    /** URL per la connessione alla base di dati. &Egrave; formato dai
     * seguenti componenti:<br>
     * <protocollo>://<host del server>/<nome base di dati>.
     */
    private String url = "jdbc:postgresql://sqlserver.sci.univr.it/dblab200";
    /** Driver da utilizzare per la connessione e l'esecuzione delle query. */
    private String driver = "org.postgresql.Driver";
    
    /** Interrogazione SQL per ottenere tutti i generi presenti 
	nella base di dati. */
    private String mostreSql = 
	"SELECT * FROM mostra";

    /**
     * Costruttore della classe. Carica i driver da utilizzare per la
     * connessione alla base di dati.
     *
     * @throws ClassNotFoundException Eccezione generata nel caso in cui
     *         i driver per la connessione non siano trovati nel
     *         CLASSPATH.
     */
    public DBMS() throws ClassNotFoundException {
	Class.forName(driver);
    }

    /**
     * Questo metodo viene utilizzato per la creazione di un MostraBean in base al
     * contenuto della riga corrente del ResultSet.
     *
     * @param rs ResultSet da cui estrarre le informazioni per la creazione del
     *           MostraBean.
     *
     * @return MostraBean creato in base al contenuto della riga corrente del
     *         ResultSet.
     *
     * @throws SQLException Eccezione generata nel caso in cui la ricerca delle
     *                      informazioni nel ResultSet non sia corretta.
     */
    private MostraBean makeMostraBean(ResultSet rs) throws SQLException {
	MostraBean bean = new MostraBean();

	bean.setTitoloMostra(rs.getString("titolo_mostra"));
	bean.setMuseo(rs.getString("nome_museo"));
        bean.setCitta(rs.getString("citta_museo"));
	bean.setInizioEsposizione(rs.getDate("inizio_esposizione"));
	bean.setFineEsposizione(rs.getDate("fine_esposizione"));
        bean.setPrezzoIntero(rs.getInt("prezzo_intero"));
	bean.setPrezzoRidotto(rs.getInt("prezzo_ridotto"));
	/*
	  IMPORTANTE!!!
	  
	  Nel caso in cui la colonna della tabella interrogata possa contenere
	  valori nulli DEVE essere verificata la presenza del valore e nel caso
	  in cui sia NULL si deve impostare il valore del campo relativo del bean
	  ad un valore di default.
	  
	  Per esempio: se la colonna PrezzoRidotto della tabella genere potesse avere
	  valori nulli il codice completo sarebbe:

	  bean.setPrezzoRidotto(rs.getInt("prezzo_ridotto"));
	  if (rs.wasNull()) bean.setPrezzoRidotto("");
	 */

	return bean;
    }

    /**
     * Questo metodo permette di ottenere tutti le mostre presenti nella tabella
     * MOSTRA.
     *
     * @return Oggetto Vector contenente una serie di MostraBean contenenti le
     *         informazioni relative.
     */
    public Vector getMostre() {
	// Dichiarazione delle variabili
	Connection con = null;
	Statement stmt = null;
	ResultSet rs = null;
	Vector result = new Vector();

	try {
	    // Tentativo di connessione al database
	    con = DriverManager.getConnection(url, user, passwd);
	    // Connessione riuscita, ottengo l'oggetto per l'esecuzione
	    // dell'interrogazione.
	    stmt = con.createStatement();
	    // Eseguo l'interrogazione desiderata
	    rs = stmt.executeQuery(mostreSql);
	    // Memorizzo il risultato dell'interrogazione nel Vector
	    while(rs.next())
		result.add(makeMostraBean(rs));
	} catch(SQLException sqle) {                /* Catturo le eventuali eccezioni! */
	    sqle.printStackTrace();
	} finally {                                 /* Alla fine chiudo la connessione. */
	    try {
		con.close();
	    } catch(SQLException sqle1) {
		sqle1.printStackTrace();
	    }
	}
	// Ritorno il Vector
	return result;
    }
}
